/**
 * <copyright>
 *
 * Copyright (c) 2002-2003 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *   IBM - Initial API and implementation
 *
 * </copyright>
 */
package org.eclipse.wsdl20.model.impl;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.eclipse.wsdl20.ValidatableElement;
import org.eclipse.wsdl20.model.AttributeInfoItem;
import org.eclipse.wsdl20.model.ElementInfoItem;

/**
 * An implementation of an element information item from the WSDL 2.0 specification.
 * 
 * @author Lawrence Mandel, IBM
 */
public class ElementInfoItemImpl implements ElementInfoItem, ValidatableElement
{
  
  protected String name = null;
  protected QName qname = null;
  protected String targetNamespace = null;
  protected Map attributeInfoItems = new Hashtable();
  protected List elementInfoItems = new Vector();
  protected String uri = null;
  protected int line = 0;
  protected int column = 0;

  /**
   * @see org.eclipse.wsdl20.model.ElementInfoItem#getName()
   * @see #setName
   */
  public String getName()
  {
    if(name == null)
    {
      name = getAttributeInfoItemValue(Constants.NS_URI_EMPTY, Constants.ATTR_NAME);
    }
    return name;
  }
  
  /**
   * @see org.eclipse.wsdl20.model.ElementInfoItem#getTargetNamespace()
   */
  public String getTargetNamespace()
  {
    if(targetNamespace == null)
    {
      // if we haven't already, retrieve the target namespace from the attribute info items
      targetNamespace = getAttributeInfoItemValue(Constants.NS_URI_EMPTY, Constants.ATTR_TARGET_NAMESPACE);
      if(targetNamespace == null)
      {
        targetNamespace = "";
      }
    }
    return targetNamespace;
  }

  /**
   * @see org.eclipse.wsdl20.model.ElementInfoItem#getAttributeInfoItems()
   */
  public AttributeInfoItem[] getAttributeInfoItems()
  {
    return (AttributeInfoItem[])attributeInfoItems.values().toArray(new AttributeInfoItem[attributeInfoItems.size()]);
  }
  
  /**
   * Get the value of an attribute info item from the namespace and local name.
   * 
   * @param namespace The namespace of the attribute info item.
   * @param localname The local name of the attribute info item.
   * @return The value of the attribute info item.
   */
  protected String getAttributeInfoItemValue(String namespace, String localname)
  { 
    if(namespace == null)
    {
      namespace = "";
    }
    StringBuffer key = new StringBuffer();
    key.append(namespace).append(":").append(localname);
    AttributeInfoItem aii = (AttributeInfoItem)attributeInfoItems.get(key.toString());
    if(aii != null)
    {
      return aii.getValue();
    }
    return null;
  }

  /**
   * @see org.eclipse.wsdl20.model.ElementInfoItem#getElementInfoItems()
   */
  public ElementInfoItem[] getElementInfoItems()
  {
    return (ElementInfoItem[])elementInfoItems.toArray(new ElementInfoItem[elementInfoItems.size()]);
  }

  /**
   * Set the column location of this element.
   * 
   * @param column The column location of this element.
   * @see #getColumn
   */
  public void setColumn(int column)
  {
    this.column = column;
  }
  
  /**
   * @see org.eclipse.validate.wsdl20.ValidatableElement#getColumn()
   * @see #setColumn
   */
  public int getColumn()
  {
    return column;
  }

  /**
   * Set the line location of this element.
   * 
   * @param line The line location of this element.
   * @see #getLine
   */
  public void setLine(int line)
  {
    this.line = line;
  }
  
  /**
   * @see org.eclipse.validate.wsdl20.ValidatableElement#getLine()
   * @see #setLine
   */
  public int getLine()
  {
    return line;
  }

  /**
   * The location uri of this element tells which file this element is in.
   * 
   * @param locationURI The URI of the file this element is located in.
   * @see #getLocationURI
   */
  public void setLocationURI(String locationURI)
  {
    this.uri = locationURI;
  }
  
  /**
   * @see org.eclipse.validate.wsdl20.ValidatableElement#getLocationURI()
   * @see #setLocation URI
   */
  public String getLocationURI()
  {
    return uri;
  }
  
  /**
   * Add an attribute info item to this element.
   * 
   * @param aii The attribute info item to add to the element.
   */
  public void addAttributeInfoItem(AttributeInfoItem aii)
  {
    if(aii != null)
    {
      StringBuffer key = new StringBuffer();
      key.append(aii.getNamespace()).append(":").append(aii.getLocalName());  
      attributeInfoItems.put(key.toString(), aii);
    }
  	
  }
  
  /**
   * Add an element info item to this element.
   * 
   * @param eii The element info item to add to the element.
   */
  public void addElementInfoItem(ElementInfoItem eii)
  {
  	elementInfoItems.add(eii);
  }

  /**
   * @see org.eclipse.wsdl20.model.ElementInfoItem#getQName()
   */
  public QName getQName()
  {
    return qname;
  }
  
  /**
   * Set the QName of this element from a namespace and local name.
   * 
   * @param namespace The namespace to set for the QName.
   * @param localname The local name to set for the QName.
   */
  public void setQName(String namespace, String localname)
  {
    this.qname = new QName(namespace, localname);
  }
  
  /**
   * Set the QName of this element.
   * 
   * @param qname The QName to set for this element.
   */
  public void setQName(QName qname)
  {
    this.qname = qname;
  }

}
