/**
 * <copyright>
 *
 * Copyright (c) 2002-2003 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *   IBM - Initial API and implementation
 *
 * </copyright>
 */
package org.eclipse.wsdl20.model.xml.impl;

import java.util.List;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.apache.xerces.dom.ElementImpl;
import org.apache.xerces.parsers.DOMParser;
import org.eclipse.wsdl20.model.AttributeInfoItem;
import org.eclipse.wsdl20.model.Binding;
import org.eclipse.wsdl20.model.BindingFaultReference;
import org.eclipse.wsdl20.model.BindingMessageReference;
import org.eclipse.wsdl20.model.BindingOperation;
import org.eclipse.wsdl20.model.Definitions;
import org.eclipse.wsdl20.model.ElementInfoItem;
import org.eclipse.wsdl20.model.FaultReference;
import org.eclipse.wsdl20.model.Feature;
import org.eclipse.wsdl20.model.Import;
import org.eclipse.wsdl20.model.Include;
import org.eclipse.wsdl20.model.Interface;
import org.eclipse.wsdl20.model.InterfaceOperation;
import org.eclipse.wsdl20.model.MessageReference;
import org.eclipse.wsdl20.model.Property;
import org.eclipse.wsdl20.model.PropertyConstraint;
import org.eclipse.wsdl20.model.PropertyValue;
import org.eclipse.wsdl20.model.Service;
import org.eclipse.wsdl20.model.ServiceEndpoint;
import org.eclipse.wsdl20.model.Types;
import org.eclipse.wsdl20.model.exception.WSDLException;
import org.eclipse.wsdl20.model.impl.AttributeInfoItemImpl;
import org.eclipse.wsdl20.model.impl.BindingFaultReferenceImpl;
import org.eclipse.wsdl20.model.impl.BindingImpl;
import org.eclipse.wsdl20.model.impl.BindingMessageReferenceImpl;
import org.eclipse.wsdl20.model.impl.BindingOperationImpl;
import org.eclipse.wsdl20.model.impl.Constants;
import org.eclipse.wsdl20.model.impl.DefinitionsImpl;
import org.eclipse.wsdl20.model.impl.ElementInfoItemImpl;
import org.eclipse.wsdl20.model.impl.FaultReferenceImpl;
import org.eclipse.wsdl20.model.impl.FeatureImpl;
import org.eclipse.wsdl20.model.impl.ImportImpl;
import org.eclipse.wsdl20.model.impl.IncludeImpl;
import org.eclipse.wsdl20.model.impl.InterfaceImpl;
import org.eclipse.wsdl20.model.impl.InterfaceOperationImpl;
import org.eclipse.wsdl20.model.impl.MessageReferenceImpl;
import org.eclipse.wsdl20.model.impl.PropertyConstraintImpl;
import org.eclipse.wsdl20.model.impl.PropertyImpl;
import org.eclipse.wsdl20.model.impl.PropertyValueImpl;
import org.eclipse.wsdl20.model.impl.ServiceEndpointImpl;
import org.eclipse.wsdl20.model.impl.ServiceImpl;
import org.eclipse.wsdl20.model.impl.TypesImpl;
import org.eclipse.wsdl20.model.xml.WSDLReader;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;

/**
 * An implementation of a WSDL reader. This will create the WSDL 2.0 model.
 * 
 * @author Lawrence Mandel, IBM
 */
public class WSDLReaderImpl implements WSDLReader
{
  private final String _ERROR_ROOT_ELEMENT_NOT_DEFINITIONS = "_ERROR_ROOT_ELEMENT_NOT_DEFINITIONS";
  private final String _ERROR_MORE_THAN_ONE_TYPES_ELEMENT = "_ERROR_MORE_THAN_ONE_TYPES_ELEMENT";
  private final String TARGETNAMESPACE = "targetNamespace";

  private List readerErrors = new Vector();
  private ResourceBundleHelper resourceBundleHelper = null;

  /**
   * Returns the resource bundle helper for this reader.
   * 
   * @return The resource bundle helper for this reader.
   */
  protected ResourceBundleHelper getResourceBundleHelper()
  {
    if(resourceBundleHelper == null)
    {  
      ResourceBundle resourcebundle = ResourceBundle.getBundle(Constants.RB_BASE_NAME);
      resourceBundleHelper = new ResourceBundleHelper(resourcebundle);
    }
    return resourceBundleHelper;
  }

  /**
   * @see org.eclipse.wsdl20.model.xml.WSDLReader#readWSDL(java.lang.String)
   */
  public Definitions readWSDL(String uri) throws WSDLException
  {
    try
    {
      Document doc = getDocument(uri);
      return readWSDL(uri, doc);
    }
    catch (WSDLException e)
    {
      throw e;
    }
  }

  /**
   * @see org.eclipse.wsdl20.model.xml.WSDLReader#readWSDL(org.w3c.dom.Document)
   */
  public Definitions readWSDL(String uri, Document document) throws WSDLException
  {
    Element elem = document.getDocumentElement();
    return parseDefinitions(elem, uri);
  }

  /**
   * Create a DOM model of an object from its uri.
   * 
   * @param uri The URI of the file.
   * @return A document representation of the file.
   * @throws WSDLException
   */
  protected Document getDocument(String uri) throws WSDLException
  {
    try
    {
      DOMParser builder = new LineNumberDOMParser();
      builder.parse(uri);
      Document doc = builder.getDocument();
      return doc;
    }
    catch (Exception e)
    {
      throw new WSDLException(e.getMessage());
    }
  }

  /**
   * Parse the element given, create a definitions element from it and its children and return
   * the definitions element.
   * 
   * @param defelem The element to parse.
   * @param uri The URI of the file that contains this element.
   * @return A WSDL 2.0 definitions element.
   * @throws WSDLException If unable to create the definitions element.
   */
  protected Definitions parseDefinitions(Element defelem, String uri) throws WSDLException
  {
    String localname = defelem.getLocalName();
    String namespace = defelem.getNamespaceURI();

    QName defQName = new QName(namespace, localname);

    // get the location information of the definitions element
    int line = 0;
    int column = 0;
    ElementImpl elementImpl = (ElementImpl)defelem;
    ElementLocation elementLocation = (ElementLocation)elementImpl.getUserData(ElementLocation.ID);
    if (elementLocation != null)
    {
      line = elementLocation.getLineNumber();
      column = elementLocation.getColumnNumber();
    }

    // ensure that this element is a definitions element
    if (!ReaderUtils.qnameEquals(defQName, Constants.Q_ELEM_DEFINITIONS))
    {
      String message = getResourceBundleHelper().getString(_ERROR_ROOT_ELEMENT_NOT_DEFINITIONS);
      readerErrors.add(new ReaderErrorImpl(message, line, column, uri));
      throw new WSDLException(message);
    }

    DefinitionsImpl def = new DefinitionsImpl();
    def.setLocationURI(uri);
    def.setLine(line);
    def.setColumn(column);

    setAttributeInfoItems(def, defelem);
   
    Element childelement = null;
    List imports = new Vector();
    List includes = new Vector();
    List types = new Vector();
    List interfaces = new Vector();
    List bindings = new Vector();
    List services = new Vector();

    NodeList childnodes = defelem.getChildNodes();
    int numChildren = childnodes.getLength();
    
    for(int i = 0; i < numChildren; i++)
    {  
      childelement = ReaderUtils.getElement(childnodes.item(i));
      QName childQName = new QName(childelement.getNamespaceURI(), childelement.getLocalName());
      
      // import element
      if(ReaderUtils.qnameEquals(childQName,Constants.Q_ELEM_IMPORT))
      {
        imports.add(childelement);
      }
      // include element
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_INCLUDE))
      {
        includes.add(childelement);
      }
      // types element
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_TYPES))
      {
        types.add(childelement);
      }
      // interface element
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_INTERFACE))
      {
        interfaces.add(childelement);
      }
      // binding element
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_BINDING))
      {
        bindings.add(childelement);
      }
      // service element
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_SERVICE))
      {
        services.add(childelement);
      }
      else
      {
        def.addElementInfoItem(parseElementInfoItem(childelement, uri));
      }
        
    }
    
    childelement = null;
      
    // add the imports
    numChildren = imports.size();
    for (int i = 0; i < numChildren; i++)
    {
      childelement = (Element)imports.get(i);
      if(childelement != null)
      {  
        def.addImport(parseImport(childelement, uri));
      }
    }

    // add the includes
    numChildren = includes.size();
    for (int i = 0; i < numChildren; i++)
    {
      childelement = (Element)includes.get(i);
      if(childelement != null)
       {  
        def.addInclude(parseInclude(childelement, uri));
      }
    }

    // add the types
    numChildren = types.size();
    if (numChildren == 1)
    {
      childelement = (Element)types.get(0);
      if(childelement != null)
       {  
        def.setTypes(parseTypes(childelement, uri));
      }
    }
    // if there is more then one types section there is an error
    else if (numChildren > 1)
    {
      for (int i = 1; i < numChildren; i++)
      {
        childelement = (Element)types.get(i);
        if(childelement != null)
         {  
          addReaderError(_ERROR_MORE_THAN_ONE_TYPES_ELEMENT, childelement, uri);
        }
        
      }
    }

    // add the interfaces
    numChildren = interfaces.size();
    for (int i = 0; i < numChildren; i++)
    {
      childelement = (Element)interfaces.get(i);
      if(childelement != null)
       {  
        def.addInterface(parseInterface(childelement, uri));
      }
    }

    // add the bindings
    numChildren = bindings.size();
    for (int i = 0; i < numChildren; i++)
    {
      childelement = (Element)bindings.get(i);
      if(childelement != null)
       {  
        def.addBinding(parseBinding(childelement, uri));
      }
    }

    // add the services
    numChildren = services.size();
    for (int i = 0; i < numChildren; i++)
    {
      childelement = (Element)services.get(i);
      if(childelement != null)
       {  
        def.addService(parseService(childelement, uri));
      }
    }

    return def;

  }
  
  /**
   * Parse the element given, create an import element from it and return the import element.
   * 
   * @param importelem The element to parse.
   * @param uri The uri of the file with this element.
   * @return An import element created from the importelem given.
   */
  protected Import parseImport(Element importelem, String uri)
  { 
    ImportImpl importimpl = new ImportImpl();
    setLocationInfo(importimpl, importelem, uri);
    
    setAttributeInfoItems(importimpl, importelem);
    
    NodeList nodelist = importelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
    {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
      {
        importimpl.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return importimpl;
  }
  
  /**
   * Parse the element given, create an include element from it and return the include element.
   * 
   * @param includeelem The element to parse.
   * @param uri The uri of the file with this element.
   * @return An include element created from the includeelem given.
   */
  protected Include parseInclude(Element includeelem, String uri)
  { 
    IncludeImpl include = new IncludeImpl();
    setLocationInfo(include, includeelem, uri);
    
    setAttributeInfoItems(include, includeelem);
    
    NodeList nodelist = includeelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
     {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
       {
        include.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return include;
    
  }
  
  /**
   * Parse the DOM element given, create a WSDL 2.0 types element from it and return the
   * types element.
   * 
   * @param typeselem The DOM element to parse.
   * @param uri The uri of the file with this element.
   * @return A WSDL 2.0 types element.
   */
  protected Types parseTypes(Element typeselem, String uri)
  {
    TypesImpl types = new TypesImpl();
    setLocationInfo(types, typeselem, uri);
    
    setAttributeInfoItems(types, typeselem);
    
    NodeList nodelist = typeselem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
     {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
       {
        types.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return types;
  }
  
  /**
   * Parse the DOM element given, create a WSDL 2.0 interface element from it and create operation,
   * feature and property children.
   * 
   * @param interfaceelem The DOM element to parse.
   * @param uri The uri of the file with this element.
   * @return A WSDL 2.0 interface element.
   */
  protected Interface parseInterface(Element interfaceelem, String uri)
  { 
    InterfaceImpl interfaceImpl = new InterfaceImpl();
    setLocationInfo(interfaceImpl, interfaceelem, uri);
    
    setAttributeInfoItems(interfaceImpl, interfaceelem);
    
    Element childelement = null;
    NodeList childnodes = interfaceelem.getChildNodes();
    int numChildren = childnodes.getLength();
    
    for(int i = 0; i < numChildren; i++)
    {
      childelement = ReaderUtils.getElement(childnodes.item(i));
      QName childQName = new QName(childelement.getNamespaceURI(), childelement.getLocalName());
      // add an operation
      if(ReaderUtils.qnameEquals(childQName,Constants.Q_ELEM_OPERATION))
      {  
        interfaceImpl.addOperation(parseInterfaceOperation(childelement, uri));
      }
      // add a feature
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_FEATURE))
      {  
        interfaceImpl.addFeature(parseFeature(childelement, uri));
      }
      // add a property
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_PROPERTY))
      {  
        interfaceImpl.addProperty(parseProperty(childelement, uri));
      }
      // add an element info item
      else
      {
        interfaceImpl.addElementInfoItem(parseElementInfoItem(childelement, uri));
      }
    }
    
    return interfaceImpl;
  }
  
  /**
   * Parse the given DOM element, create a WSDL 2.0 interface operation from the DOM
   * element and children elements and return it.
   * 
   * @param operelem The DOM element to create the interface element from.
   * @param uri The uri of the file the element is found in.
   * @return A WSDL 2.0 interface operation element.
   */
  protected InterfaceOperation parseInterfaceOperation(Element operelem, String uri)
  {
    InterfaceOperationImpl interfaceOperation = new InterfaceOperationImpl();
    setLocationInfo(interfaceOperation, operelem, uri);
    
    setAttributeInfoItems(interfaceOperation, operelem);
    
    Element childelement = null;
    NodeList childnodes = operelem.getChildNodes();
    int numChildren = childnodes.getLength();
    
    for(int i = 0; i < numChildren; i++)
    {  
      childelement = ReaderUtils.getElement(childnodes.item(i));
      QName childQName = new QName(childelement.getNamespaceURI(), childelement.getLocalName());
    
      // add an input message reference
      if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_INPUT))
      {  
        interfaceOperation.addMessageReference(parseInput(childelement, uri));
      }
      // add an output message reference
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_OUTPUT))
      {  
        interfaceOperation.addMessageReference(parseOutput(childelement, uri));
      }
      // add an infault fault reference
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_INFAULT))
      {  
        interfaceOperation.addFaultReference(parseInfault(childelement, uri));
      }
      // add an outfault fault reference
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_OUTFAULT))
      {  
        interfaceOperation.addFaultReference(parseOutfault(childelement, uri));
      }
      // add a feature
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_FEATURE))
      {  
        interfaceOperation.addFeature(parseFeature(childelement, uri));
      }
      // add a property
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_PROPERTY))
      {  
        interfaceOperation.addProperty(parseProperty(childelement, uri));
      }
      // add an element info item
      else
      {
        interfaceOperation.addElementInfoItem(parseElementInfoItem(childelement, uri));
      }
    }
    
    return interfaceOperation;
    
  }
  
  /**
   * Parse the given DOM element, create a WSDL 2.0 message reference from the DOM
   * element and children elements and return it.
   * 
   * @param inputelem The DOM element to parse.
   * @param uri The URI of the file the element is located in.
   * @return A message reference representing the input element.
   */
  public MessageReference parseInput(Element inputelem, String uri)
  {
    MessageReferenceImpl messageref = new MessageReferenceImpl();
    messageref.setQName(Constants.Q_ELEM_INPUT);
    
    setLocationInfo(messageref, inputelem, uri);
    
    setAttributeInfoItems(messageref, inputelem);
    
    NodeList nodelist = inputelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
     {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
       {
        messageref.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return messageref;
  }
  
  /**
   * Parse the given DOM element, create a WSDL 2.0 message reference from the DOM
   * element and children elements and return it.
   * 
   * @param outputelem The DOM element to parse.
   * @param uri The URI of the file the element is located in.
   * @return A message reference representing the output element.
   */
  public MessageReference parseOutput(Element outputelem, String uri)
  {
    MessageReferenceImpl messageref = new MessageReferenceImpl();
    messageref.setQName(Constants.Q_ELEM_OUTPUT);
    
    setLocationInfo(messageref, outputelem, uri);
    
    setAttributeInfoItems(messageref, outputelem);
    
    NodeList nodelist = outputelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
     {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
       {
        messageref.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return messageref;
  }
  
  /**
   * Parse the given DOM element, create a WSDL 2.0 fault reference from the DOM
   * element and children elements and return it.
   * 
   * @param infaultelem The DOM element to parse.
   * @param uri The URI of the file the element is located in.
   * @return A message reference representing the infault element.
   */
  public FaultReference parseInfault(Element infaultelem, String uri)
  {
    FaultReferenceImpl faultref = new FaultReferenceImpl();
    faultref.setQName(Constants.Q_ELEM_INFAULT);
    
    setLocationInfo(faultref, infaultelem, uri);
    
    setAttributeInfoItems(faultref, infaultelem);
    
    NodeList nodelist = infaultelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
     {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
       {
        faultref.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return faultref;
  }
  
  /**
   * Parse the given DOM element, create a WSDL 2.0 fault reference from the DOM
   * element and children elements and return it.
   * 
   * @param infaultelem The DOM element to parse.
   * @param uri The URI of the file the element is located in.
   * @return A message reference representing the outfault element.
   */
  public FaultReference parseOutfault(Element outfaultelem, String uri)
  {
    FaultReferenceImpl faultref = new FaultReferenceImpl();
    faultref.setQName(Constants.Q_ELEM_OUTFAULT);
    
    setLocationInfo(faultref, outfaultelem, uri);
    
    setAttributeInfoItems(faultref, outfaultelem);
    
    NodeList nodelist = outfaultelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
     {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
       {
        faultref.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return faultref;
  }
  
  /**
   * Parse the given DOM element, create a WSDL 2.0 binding from the DOM
   * element and children elements and return it.
   * 
   * @param bindingelem The DOM element to parse.
   * @param uri The URI of the file the element is located in.
   * @return A WSDL 2.0 binding element.
   */
  public Binding parseBinding(Element bindingelem, String uri)
  {
    BindingImpl binding = new BindingImpl();
    setLocationInfo(binding, bindingelem, uri);
    
    setAttributeInfoItems(binding, bindingelem);
    
    Element childelement = null;
    NodeList childnodes = bindingelem.getChildNodes();
    int numChildren = childnodes.getLength();
    
    for(int i = 0; i < numChildren; i++)
    {
      childelement = ReaderUtils.getElement(childnodes.item(i));
      QName childQName = new QName(childelement.getNamespaceURI(), childelement.getLocalName());
      
      // add a binding operation
      if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_OPERATION))
      {
        binding.addBindingOperation(parseBindingOperation(childelement, uri));
      }
      // add a feature
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_FEATURE))
      {
        binding.addFeature(parseFeature(childelement, uri));
      }
      // add a property
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_PROPERTY))
      {
        binding.addProperty(parseProperty(childelement, uri));
      }
      // add an element info item
      else
      {
        binding.addElementInfoItem(parseElementInfoItem(childelement, uri));
      }
    }
    
    return binding;
  }
  
  /**
   * Parse the given DOM element, create a binding operation element and its children
   * elements and return the binding operation element.
   *  
   * @param boelem The DOM element to parse.
   * @param uri The URI of the file that contains the element.
   * @return A WSDL 2.0 binding operation element.
   */
  protected BindingOperation parseBindingOperation(Element boelem, String uri)
  {
    BindingOperationImpl bindingoperation = new BindingOperationImpl();
    setLocationInfo(bindingoperation, boelem, uri);
    
    setAttributeInfoItems(bindingoperation, boelem);
    
    Element childelement = null;
    NodeList childnodes = boelem.getChildNodes();
    int numChildren = childnodes.getLength();
    
    for(int i = 0; i < numChildren; i++)
    {
      childelement = ReaderUtils.getElement(childnodes.item(i));
      QName childQName = new QName(childelement.getNamespaceURI(), childelement.getLocalName());
      
      // add an input binding message reference
      if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_INPUT))
      {
        bindingoperation.addBindingMessageReference(parseBindingInput(childelement, uri));
      }
      // add an output binding message reference
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_OUTPUT))
      {
        bindingoperation.addBindingMessageReference(parseBindingOutput(childelement, uri));
      }
      // add an infault binding fault reference
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_INFAULT))
      {
        bindingoperation.addBindingFaultReference(parseBindingInfault(childelement, uri));
      }
      // add an outfault binding fault reference
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_OUTFAULT))
      {
        bindingoperation.addBindingFaultReference(parseBindingOutfault(childelement, uri));
      }
      // add a feature
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_FEATURE))
      {
        bindingoperation.addFeature(parseFeature(childelement, uri));
      }
      // add a property
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_PROPERTY))
      {
        bindingoperation.addProperty(parseProperty(childelement, uri));
      }
      // add an element info item
      else
      {
        bindingoperation.addElementInfoItem(parseElementInfoItem(childelement, uri));
      }
    }
    
    return bindingoperation;
  }
  
  /**
   * Parse the given DOM element, create a WSDL 2.0 binding message reference from the DOM
   * element and children elements and return it.
   * 
   * @param inputelem The DOM element to parse.
   * @param uri The URI of the file the element is located in.
   * @return A binding message reference representing the input element.
   */
  public BindingMessageReference parseBindingInput(Element inputelem, String uri)
  {
    BindingMessageReferenceImpl bindingmessageref = new BindingMessageReferenceImpl();
    bindingmessageref.setQName(Constants.Q_ELEM_INPUT);
    
    setLocationInfo(bindingmessageref, inputelem, uri);
    
    setAttributeInfoItems(bindingmessageref, inputelem);
    
    NodeList nodelist = inputelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
    {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
      {
        bindingmessageref.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return bindingmessageref;
  }
  
  /**
   * Parse the given DOM element, create a WSDL 2.0 binding message reference from the DOM
   * element and children elements and return it.
   * 
   * @param outputelem The DOM element to parse.
   * @param uri The URI of the file the element is located in.
   * @return A binding message reference representing the output element.
   */
  public BindingMessageReference parseBindingOutput(Element outputelem, String uri)
  {
    BindingMessageReferenceImpl bindingmessageref = new BindingMessageReferenceImpl();
    bindingmessageref.setQName(Constants.Q_ELEM_OUTPUT);
    
    setLocationInfo(bindingmessageref, outputelem, uri);
    
    setAttributeInfoItems(bindingmessageref, outputelem);
    
    NodeList nodelist = outputelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
    {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
      {
        bindingmessageref.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return bindingmessageref;
  }
  
  /**
   * Parse the given DOM element, create a WSDL 2.0 binding fault reference from the DOM
   * element and children elements and return it.
   * 
   * @param infaultelem The DOM element to parse.
   * @param uri The URI of the file the element is located in.
   * @return A binding fault reference representing the infault element.
   */
  public BindingFaultReference parseBindingInfault(Element infaultelem, String uri)
  {
    BindingFaultReferenceImpl bindingfaultref = new BindingFaultReferenceImpl();
    bindingfaultref.setQName(Constants.Q_ELEM_INFAULT);
    
    setLocationInfo(bindingfaultref, infaultelem, uri);
    
    setAttributeInfoItems(bindingfaultref, infaultelem);
    
    NodeList nodelist = infaultelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
     {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
       {
        bindingfaultref.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return bindingfaultref;
  }
  
  /**
   * Parse the given DOM element, create a WSDL 2.0 binding fault reference from the DOM
   * element and children elements and return it.
   * 
   * @param outfaultelem The DOM element to parse.
   * @param uri The URI of the file the element is located in.
   * @return A binding fault reference representing the outfault element.
   */
  public BindingFaultReference parseBindingOutfault(Element outfaultelem, String uri)
  {
    BindingFaultReferenceImpl bindingfaultref = new BindingFaultReferenceImpl();
    bindingfaultref.setQName(Constants.Q_ELEM_OUTFAULT);
    
    setLocationInfo(bindingfaultref, outfaultelem, uri);
    
    setAttributeInfoItems(bindingfaultref, outfaultelem);
    
    NodeList nodelist = outfaultelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
     {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
       {
        bindingfaultref.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return bindingfaultref;
  }
  
  /**
   * Parse the given DOM element, create a WSDL 2.0 feature from the DOM
   * element and children elements and return it.
   * 
   * @param featureelem The DOM element to create the feature from.
   * @param uri The uri of the file the element is found in.
   * @return A WSDL 2.0 feature element.
   */
  protected Feature parseFeature(Element featureelem, String uri)
  {
    FeatureImpl feature = new FeatureImpl();
    setLocationInfo(feature, featureelem, uri);
    
    setAttributeInfoItems(feature, featureelem);
    
    NodeList childnodes = featureelem.getChildNodes();
    int numChildren = childnodes.getLength();
    
    for(int i = 0; i < numChildren; i++)
    {
      Element elem = ReaderUtils.getElement(childnodes.item(i));
      if(elem != null)
      {
        feature.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return feature;
  }
  
  /**
   * Parse a DOM element, create a WSDL 2.0 property element and return the property element.
   * 
   * @param propelem The DOM element to parse.
   * @param uri The URI of the file that contains the element.
   * @return A property element.
   */
  protected Property parseProperty(Element propelem, String uri)
  {
    PropertyImpl property = new PropertyImpl();
    setLocationInfo(property, propelem, uri);
    
    setAttributeInfoItems(property, propelem);
    
    Element childelement = null;
    NodeList childnodes = propelem.getChildNodes();
    int numChildren = childnodes.getLength();
    for(int i = 0; i < numChildren; i++)
    {
      childelement = ReaderUtils.getElement(childnodes.item(i));
      QName childQName = new QName(childelement.getNamespaceURI(), childelement.getLocalName());
      // add a value
      if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_VALUE))
      {
        property.setValue(parseValue(childelement, uri));
      }
      // add a contraint
      else if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_CONSTRAINT))
      {
        property.setConstraint(parseConstraint(childelement, uri));
      }
      // add an element info item
      else
      {
        property.addElementInfoItem(parseElementInfoItem(childelement, uri));
      }
    }
    
    return property;
  }
  
  /**
   * Parse a DOM element, create a WSDL 2.0 property value element and return the 
   * property value element.
   * 
   * @param valueelem The DOM element to parse.
   * @param uri The URI of the file that contains the element.
   * @return A property value element.
   */
  protected PropertyValue parseValue(Element valueelem, String uri)
  {
    PropertyValueImpl value = new PropertyValueImpl();
    setLocationInfo(value, valueelem, uri);
    
    setAttributeInfoItems(value, valueelem);
    // TODO: get the value of the value
    
    NodeList nodelist = valueelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
     {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
       {
        value.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return value;
    
  }
  
  /**
   * Parse a DOM element, create a WSDL 2.0 property constraint element and return the 
   * property constraint element.
   * 
   * @param valueelem The DOM element to parse.
   * @param uri The URI of the file that contains the element.
   * @return A property constraint element.
   */
  protected PropertyConstraint parseConstraint(Element constraintelem, String uri)
  {
    PropertyConstraintImpl constraint = new PropertyConstraintImpl();
    setLocationInfo(constraint, constraintelem, uri);
    
    setAttributeInfoItems(constraint, constraintelem);
    // TODO: get the value of the constraint
    
    NodeList nodelist = constraintelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
     {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
       {
        constraint.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return constraint;
    
  }
  
  /**
   * Parse a DOM element, create a WSDL 2.0 service element and return the 
   * service element.
   * 
   * @param serviceelem The DOM element to parse.
   * @param uri The URI of the file that contains the element.
   * @return A service element.
   */
  protected Service parseService(Element serviceelem, String uri)
  {
    ServiceImpl service = new ServiceImpl();
    setLocationInfo(service, serviceelem, uri);
    
    setAttributeInfoItems(service, serviceelem);
    
    Element childelement = null;
    NodeList childnodes = serviceelem.getChildNodes();
    int numChildren = childnodes.getLength();
    for(int i = 0; i < numChildren; i++)
     {
      childelement = ReaderUtils.getElement(childnodes.item(i));
      QName childQName = new QName(childelement.getNamespaceURI(), childelement.getLocalName());
      // add an endpoint
      if(ReaderUtils.qnameEquals(childQName, Constants.Q_ELEM_ENDPOINT))
      {
        service.addEndpoint(parseEndpoint(childelement, uri));
      }
      // add an element info item
      else
       {
        service.addElementInfoItem(parseElementInfoItem(childelement, uri));
      }
    }
    
    return service;
  }
  
  /**
   * Parse a DOM element, create a WSDL 2.0 endpoint element and return the 
   * endpoint element.
   * 
   * @param endpointelem The DOM element to parse.
   * @param uri The URI of the file that contains the element.
   * @return An endpoint element.
   */
  protected ServiceEndpoint parseEndpoint(Element endpointelem, String uri)
  {
    ServiceEndpointImpl endpoint = new ServiceEndpointImpl();
    setLocationInfo(endpoint, endpointelem, uri);
    
    setAttributeInfoItems(endpoint, endpointelem);
    
    NodeList nodelist = endpointelem.getChildNodes();
    int numelems = nodelist.getLength();
    for(int i = 0; i < numelems; i++)
     {
      Element elem = ReaderUtils.getElement(nodelist.item(i));
      if(elem != null)
       {
        endpoint.addElementInfoItem(parseElementInfoItem(elem, uri));
      }
    }
    
    return endpoint;
  }
  
  /**
   * Parse the given DOM element, create a WSDL 2.0 element info item from the
   * DOM element and children elements and return it.
   * 
   * @param elem The DOM element to parse.
   * @param uri The URI of the file that contains this element.
   * @return An WSDL 2.0 element info item representing this element.
   */
  protected ElementInfoItem parseElementInfoItem(Element elem, String uri)
  {
    ElementInfoItemImpl eii = null;
    
    if(elem != null)
    {  
      eii = new ElementInfoItemImpl();
      eii.setQName(elem.getNamespaceURI(), elem.getLocalName());
      
      NodeList nodelist = elem.getChildNodes();
      int numChildren = nodelist.getLength();
      for(int i = 0; i < numChildren; i++)
      {
        Element childElem = ReaderUtils.getElement(nodelist.item(i));
        eii.addElementInfoItem(parseElementInfoItem(childElem, uri));
      }
    }
    return eii;
  }

  /**
   * Add the attributes from element as attribute info items to the element info item specified.
   * 
   * @param elementInfoItem The WSDL 2.0 element to add the attributes to.
   * @param element The dom element to get the attributes from.
   */
  protected void setAttributeInfoItems(ElementInfoItemImpl elementInfoItem, Element element)
  {
    NamedNodeMap attributes = element.getAttributes();
    int attsize = attributes.getLength();

    // create and set the attribute info items
    for (int i = 0; i < attsize; i++)
    {
      Attr attr = (Attr)attributes.item(i);
      String nsURI = attr.getNamespaceURI();
      String localName = attr.getLocalName();
      String value = attr.getValue();

      if (localName != null)
      {
        AttributeInfoItem aii = new AttributeInfoItemImpl(nsURI, localName, value);
        elementInfoItem.addAttributeInfoItem(aii);
      }
    }
  }

  /**
   * @see org.eclipse.wsdl20.model.xml.WSDLReader#getReaderErrors()
   */
  public List getReaderErrors()
  {
    return readerErrors;
  }
  
  /**
   * Add a reader error to the list of errors.
   * 
   * @param message The error message to add.
   * @param element The DOM element that contains the error.
   * @param uri The URI of the file that containst the error.
   */
  protected void addReaderError(String message, Element element, String uri)
  {
    int line = 0;
    int column = 0;
    if(element instanceof ElementImpl)
    {
      ElementImpl elementImpl = (ElementImpl)element;
      ElementLocation elementLocation = (ElementLocation)elementImpl.getUserData(ElementLocation.ID);
      if (elementLocation != null)
       {
        line = elementLocation.getLineNumber();
        column = elementLocation.getColumnNumber();
      }
    }
    
    readerErrors.add(new ReaderErrorImpl(message, line, column, uri));
  }
  
  /**
   * Set the location information from the element on the element info item.
   * 
   * @param elementInfoItem The item to set the location information on.
   * @param element The element that contains the location information.
   * @param uri The URI of the file that contains this element.
   */
  protected void setLocationInfo(ElementInfoItemImpl elementInfoItem, Element element, String uri)
  {
    int line = 0;
    int column = 0;
    ElementImpl elementImpl = (ElementImpl)element;
    ElementLocation elementLocation = (ElementLocation)elementImpl.getUserData(ElementLocation.ID);
    if (elementLocation != null)
     {
      line = elementLocation.getLineNumber();
      column = elementLocation.getColumnNumber();
    }
    elementInfoItem.setLocationURI(uri);
    elementInfoItem.setLine(line);
    elementInfoItem.setColumn(column);
  }

}
