/**
 * <copyright>
 *
 * Copyright (c) 2002-2003 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *   IBM - Initial API and implementation
 *
 * </copyright>
 */
package org.eclipse.wsdl.validate.internal;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.wsdl.validate.ValidationMessage;

/**
 * A validation message is created when there is information to report from
 * validating a WSDL document.
 * 
 * @author Lawrence Mandel, IBM
 */
public class ValidationMessageImpl implements ValidationMessage
{
  protected String message;
  protected int lineNumber;
  protected int columnNumber;
  protected String uri;
  protected int severity = SEV_ERROR;
  protected List nestedErrors;

  /**
   * Constructor.
   * 
   * @param message The validation message.
   * @param lineNumber The line where the message should be displayed.
   * @param columnNumber The column where the message should be displayed.
   * @deprecated Use contructor with URI parameter.
   */
  public ValidationMessageImpl(String message, int lineNumber, int columnNumber, int severity)
  {
    this(message, lineNumber, columnNumber, severity, null);
  }

  /**
   * Constructor.
   * Allows specifying a uri for the reference that the message refers to.
   * 
   * @param message The validation message.
   * @param lineNumber The line where the message should be displayed.
   * @param columnNumber The column where the message should be displayed.
   * @param uri The uri of the reference file for the message.
   */
  public ValidationMessageImpl(String message, int lineNumber, int columnNumber, int severity, String uri)
  {
    this.message = message;
    this.lineNumber = lineNumber;
    this.columnNumber = columnNumber;
    this.severity = severity;
    this.uri = uri;
  }

  /**
   * Returns the validation message.
   * 
   * @return The validation message.
   */
  public String getMessage()
  {
    return message;
  }

  /**
   * Returns the column number.
   * 
   * @return The column number where the message is located.
   */
  public int getColumn()
  {
    return columnNumber;
  }

  /**
   * Returns the line number.
   * 
   * @return The line number where the message is located.
   */
  public int getLine()
  {
    return lineNumber;
  }

  /**
   * returns the uri of the reference file for the validation message.
   * 
   * @return The uri of the resource that the message refers to.
   */
  public String getURI()
  {
    return uri;
  }
  /**
   * @see org.eclipse.wsdl.validate.ValidationMessage#getSeverity()
   */
  public int getSeverity()
  {
    return severity;
  }
  
  /**
   * Set the severity of the message.
   * 
   * @param severity The severity of the message.
   */
  public void setSeverity(int severity)
  {
    if(severity == SEV_ERROR || severity == SEV_WARNING)
    {
      this.severity = severity;
    }
  }
  
  /**
   * Add a nested validation message to this validation message.
   * 
   * @param validationMessage The validation message to add as a nested message.
   */
  public void addNestedMessage(ValidationMessage validationMessage)
  {
    if (nestedErrors == null)
    {
      nestedErrors = new ArrayList();
    }
    nestedErrors.add(validationMessage);
    int validaitonmessageSeverity = validationMessage.getSeverity();
    if(validaitonmessageSeverity == SEV_ERROR)
    {
      setSeverity(SEV_ERROR);
    }
  }

  /**
   * Get the list of nested validation messages.
   * 
   * @return The list of nested validation messages.
   */
  public List getNestedMessages()
  {
    return nestedErrors != null ? nestedErrors : Collections.EMPTY_LIST;
  }

}
