/**
 * <copyright>
 *
 * Copyright (c) 2002-2003 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *   IBM - Initial API and implementation
 *
 * </copyright>
 */
package org.eclipse.wsdl.validate.ui;

import java.util.ResourceBundle;

import org.eclipse.wsdl.validate.internal.ValidatorRegistry;
import org.eclipse.wsdl.validate.internal.WSDLValidatorDelegate;
import org.eclipse.wsdl.validate.wsdl11.internal.WSDL11ValidatorDelegate;
import org.eclipse.wsdl.validate.wsdl20.internal.WSDL20ValidatorDelegate;

import com.ibm.wsdl.Constants;

/**
 * A class with methods to set validators and schemas that can be used
 * by all interfaces to the validator.
 * 
 * @author Lawrence Mandel, IBM
 */
public class WSDLConfigurator
{
  /**
   * Register a WSDL validator with the given namespace.
   * 
   * @param namespace The namespace of the validator.
   * @param validatorClassname The name of the validator class to register.
   * @param resourceBundle The name of the validator base resource bundle.
   * @param classloader The classloader to use to load the validator.
   */
  public static void registerWSDLValidator(String namespace, String validatorClassname, String resourceBundle, ClassLoader classloader)
  {
    WSDLValidatorDelegate delegate = new WSDLValidatorDelegate(validatorClassname, resourceBundle, classloader);
    ValidatorRegistry.getInstance().registerValidator(namespace, delegate, ValidatorRegistry.WSDL_VALIDATOR);
  }

  /**
   * Register an extension validator with the given namespace.
   * 
   * @param namespace The namespace of the validator.
   * @param validatorClassname The name of the validator class to register.
   * @param resourceBundle The name of the validator base resource bundle.
   * @param classloader The classloader to use to load the validator.
   */
  public static void registerExtensionValidator(String namespace, String validatorClassname, String resourceBundle, ClassLoader classloader)
  {
    WSDLValidatorDelegate delegate = new WSDLValidatorDelegate(validatorClassname, resourceBundle, classloader);
    ValidatorRegistry.getInstance().registerValidator(namespace, delegate, ValidatorRegistry.EXT_VALIDATOR);
  }

  /**
   * Register a WSDL 1.1 validator with the given namespace.
   * 
   * @param namespace The namespace of the validator
   * @param validatorClassname The name of the validator class to register
   * @param resourceBundle The name of the validator base resource bundle.
   * @param classloader The classloader to use to load the validator.
   */
  public static void registerWSDL11Validator(String namespace, String validatorClassname, String resourceBundle, ClassLoader classloader)
  {
  	WSDL11ValidatorDelegate delegate = new WSDL11ValidatorDelegate(validatorClassname, resourceBundle, classloader);
    org.eclipse.wsdl.validate.wsdl11.internal.ValidatorRegistry.getInstance().registerValidator(namespace, delegate);
  }
  
  /**
   * Register a WSDL 2.0 validator with the given namespace.
   * 
   * @param namespace The namespace of the validator
   * @param validatorClassname The name of the validator class to register
   * @param resourceBundle The name of the validator base resource bundle.
   */
  public static void registerWSDL20Validator(String namespace, String validatorClassname, String resourceBundle)
  {
    WSDL20ValidatorDelegate delegate = new WSDL20ValidatorDelegate(validatorClassname, resourceBundle);
    org.eclipse.wsdl.validate.wsdl20.internal.ValidatorRegistry.getInstance().registerValidator(namespace, delegate);
  }

  /**
   * Register the default validators. Registers validators for:
   * WSDL 1.1
   * WSDL 1.1 SOAP
   * WSDL 1.1 HTTP
   * WSDL 1.1 MIME
   * 
   * @param rb - the resource bundle of the WSDL 1.1 validator
   */
  public static void registerDefaultValidators(ResourceBundle rb)
  {
    // Register the WSDL 1.1 validator controller and validators.
    registerWSDLValidator(Constants.NS_URI_WSDL, "org.eclipse.wsdl.validate.wsdl11.internal.WSDL11ValidatorController", "validatewsdl", null);
    registerWSDL11Validator(Constants.NS_URI_WSDL, "org.eclipse.wsdl.validate.wsdl11.internal.WSDL11BasicValidator", "validatewsdl", null);
    registerWSDL11Validator(
      org.eclipse.wsdl.validate.Constants.NS_HTTP, "org.eclipse.wsdl.validate.http.wsdl11.HTTPValidator","validatewsdlhttp", null);
    registerWSDL11Validator(
      org.eclipse.wsdl.validate.Constants.NS_SOAP11,"org.eclipse.wsdl.validate.soap.wsdl11.SOAPValidator","validatewsdlsoap", null);
    registerWSDL11Validator(
      org.eclipse.wsdl.validate.Constants.NS_MIME,"org.eclipse.wsdl.validate.mime.wsdl11.MIMEValidator","validatewsdlmime", null);
    
    // The WSDL 1.1 schema validator is a special case as it is registered for three namespaces.
    // We will call directly here so we can use the same delegate for all three namespaces.
    WSDL11ValidatorDelegate delegate = new WSDL11ValidatorDelegate("org.eclipse.wsdl.validate.wsdl11.internal.xsd.InlineSchemaValidator", "validatewsdl");
    org.eclipse.wsdl.validate.wsdl11.internal.ValidatorRegistry.getInstance().registerValidator(Constants.NS_URI_XSD_1999, delegate);
    org.eclipse.wsdl.validate.wsdl11.internal.ValidatorRegistry.getInstance().registerValidator(Constants.NS_URI_XSD_2000, delegate);
    org.eclipse.wsdl.validate.wsdl11.internal.ValidatorRegistry.getInstance().registerValidator(Constants.NS_URI_XSD_2001, delegate);
    
    // Register the WSDL 2.0 validator controller and validators.
    registerWSDLValidator(org.eclipse.wsdl20.model.impl.Constants.NS_URI_WSDL, "org.eclipse.wsdl.validate.wsdl20.internal.WSDL20ValidatorController", "validatewsdl", null);
    
    registerWSDL20Validator(org.eclipse.wsdl20.model.impl.Constants.NS_URI_WSDL, "org.eclipse.wsdl.validate.wsdl20.internal.WSDL20BasicValidator", "validatewsdl");
  }
}
